Index: src/sys/opencrypto/cryptosoft.c
===================================================================
--- src/sys/opencrypto/cryptosoft.c	(revision 377628)
+++ src/sys/opencrypto/cryptosoft.c	(working copy)
@@ -35,6 +35,8 @@
 #include <sys/random.h>
 #include <sys/kernel.h>
 #include <sys/uio.h>
+#include <sys/lock.h> /* Isilon bug 119136 fix */
+#include <sys/rwlock.h> /* Isilon bug 119136 fix */
 
 #include <crypto/blowfish/blowfish.h>
 #include <crypto/sha1.h>
@@ -54,6 +56,7 @@
 static	int32_t swcr_id;
 static	struct swcr_data **swcr_sessions = NULL;
 static	u_int32_t swcr_sesnum;
+static  struct rwlock swcr_sessions_lock; /* Isilon bug 119136 fix */
 
 u_int8_t hmac_ipad_buffer[HMAC_MAX_BLOCK_LEN];
 u_int8_t hmac_opad_buffer[HMAC_MAX_BLOCK_LEN];
@@ -618,6 +621,7 @@
 	if (sid == NULL || cri == NULL)
 		return EINVAL;
 
+	rw_wlock(&swcr_sessions_lock); /* Isilon bug 119136 fix */
 	if (swcr_sessions) {
 		for (i = 1; i < swcr_sesnum; i++)
 			if (swcr_sessions[i] == NULL)
@@ -640,6 +644,7 @@
 				swcr_sesnum = 0;
 			else
 				swcr_sesnum /= 2;
+			rw_wunlock(&swcr_sessions_lock); /* Isilon bug 119136 fix */
 			return ENOBUFS;
 		}
 
@@ -652,8 +657,8 @@
 
 		swcr_sessions = swd;
 	}
-
 	swd = &swcr_sessions[i];
+	rw_wunlock(&swcr_sessions_lock); /* Isilon bug 119136 fix */
 	*sid = i;
 
 	while (cri) {
@@ -700,7 +705,7 @@
 			}
 			(*swd)->sw_exf = txf;
 			break;
-	
+
 		case CRYPTO_MD5_HMAC:
 			axf = &auth_hash_hmac_md5;
 			goto authcommon;
@@ -823,13 +828,18 @@
 	struct comp_algo *cxf;
 	u_int32_t sid = CRYPTO_SESID2LID(tid);
 
+	rw_rlock(&swcr_sessions_lock); /* Isilon bug 119136 fix */
 	if (sid > swcr_sesnum || swcr_sessions == NULL ||
-	    swcr_sessions[sid] == NULL)
+	    swcr_sessions[sid] == NULL) {
+		rw_runlock(&swcr_sessions_lock); /* Isilon bug 119136 fix */
 		return EINVAL;
+	}
 
 	/* Silently accept and return */
-	if (sid == 0)
+	if (sid == 0) {
+		rw_runlock(&swcr_sessions_lock); /* Isilon bug 119136 fix */
 		return 0;
+        }
 
 	while ((swd = swcr_sessions[sid]) != NULL) {
 		swcr_sessions[sid] = swd->sw_next;
@@ -897,6 +907,7 @@
 
 		FREE(swd, M_CRYPTO_DATA);
 	}
+	rw_runlock(&swcr_sessions_lock);
 	return 0;
 }
 
@@ -920,10 +931,13 @@
 	}
 
 	lid = crp->crp_sid & 0xffffffff;
+	rw_rlock(&swcr_sessions_lock); /* Isilon bug 119136 fix */
 	if (lid >= swcr_sesnum || lid == 0 || swcr_sessions[lid] == NULL) {
+		rw_runlock(&swcr_sessions_lock); /* Isilon bug 119136 fix */
 		crp->crp_etype = ENOENT;
 		goto done;
 	}
+	rw_runlock(&swcr_sessions_lock); /* Isilon bug 119136 fix */
 
 	/* Go through crypto descriptors, processing as we go */
 	for (crd = crp->crp_desc; crd; crd = crd->crd_next) {
@@ -937,10 +951,12 @@
 		 * XXX between the various instances of an algorithm (so we can
 		 * XXX locate the correct crypto context).
 		 */
+		rw_rlock(&swcr_sessions_lock); /* Isilon bug 119136 fix */
 		for (sw = swcr_sessions[lid];
 		    sw && sw->sw_alg != crd->crd_alg;
 		    sw = sw->sw_next)
 			;
+		rw_runlock(&swcr_sessions_lock); /* Isilon bug 119136 fix */
 
 		/* No such context ? */
 		if (sw == NULL) {
@@ -1017,6 +1033,7 @@
 static int
 swcr_attach(device_t dev)
 {
+	rw_init(&swcr_sessions_lock, "swcr_sessions_lock"); /* Isilon bug 119136 fix */
 	memset(hmac_ipad_buffer, HMAC_IPAD_VAL, HMAC_MAX_BLOCK_LEN);
 	memset(hmac_opad_buffer, HMAC_OPAD_VAL, HMAC_MAX_BLOCK_LEN);
 
@@ -1057,8 +1074,11 @@
 swcr_detach(device_t dev)
 {
 	crypto_unregister_all(swcr_id);
+	rw_wlock(&swcr_sessions_lock);
 	if (swcr_sessions != NULL)
 		FREE(swcr_sessions, M_CRYPTO_DATA);
+	rw_wunlock(&swcr_sessions_lock);
+	rw_destroy(&swcr_sessions_lock);
 }
 
 static device_method_t swcr_methods[] = {
